/**
 * Copyright (c) 2016-2018 CPU and Fundamental Software Research Center, CAS
 * 
 * This software is published by the license of CPU-OS License，you can use and 
 * distibuted this software under this License. See CPU-OS Liense for more detail.
 *
 * You should have receive a copy of CPU-OS License. If not, please contact us 
 * by email <support_os@cpu-os.ac.cn>  
 * 
**/

#ifndef _CDOS_UPDATE_MANAGER_H_
#define _CDOS_UPDATE_MANAGER_H_

#include <glib.h>
#include <glib-object.h>

/**
 * SECTION:Cdos-API
 * @title: CdosUpdateManager
 * @short_description: Update manager of CDOS API.
 * 
 * #CdosUpdateManager represents a series of CDOS application update API.
 */

G_BEGIN_DECLS

#define CDOS_UPDATE_TYPE_MANAGER         (cdos_update_manager_get_type ())
#define CDOS_UPDATE_MANAGER(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), CDOS_UPDATE_TYPE_MANAGER, CdosUpdateManager))
#define CDOS_UPDATE_MANAGER_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), CDOS_UPDATE_TYPE_MANAGER, CdosUpdateManagerClass))
#define CDOS_UPDATE_IS_MANAGER(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), CDOS_UPDATE_TYPE_MANAGER))
#define CDOS_UPDATE_IS_MANAGER_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), CDOS_UPDATE_TYPE_MANAGER))
#define CDOS_UPDATE_MANAGER_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), CDOS_UPDATE_TYPE_MANAGER, CdosUpdateManagerClass))

typedef struct _CdosUpdateManager CdosUpdateManager;
typedef struct _CdosUpdateManagerClass CdosUpdateManagerClass;
typedef struct _CdosUpdateManagerPrivate CdosUpdateManagerPrivate;

struct _CdosUpdateManager
{
    /*< private >*/
    GObject               parent_object;

    CdosUpdateManagerPrivate *priv;
};

struct _CdosUpdateManagerClass
{
    GObjectClass parent_class;
};

/**
 * CdosUpdateError:
 * @CDOS_UPDATE_ERROR_BAD_VALUE: The invalid value.
 * @CDOS_UPDATE_ERROR_INIT_FAILED: Initialization error.
 * @CDOS_UPDATE_ERROR_INIT_TIMEOUT: Initialization timeout.
 * @CDOS_UPDATE_ERROR_DOWNLOAD_FAILED: A download progress callback error. 
 * @CDOS_UPDATE_ERROR_DOWNLOAD_REQUEST_FAILED: A download callback error.
 * @CDOS_UPDATE_ERROR_INSTALL_FAILED: A install progress callback error.
 * @CDOS_UPDATE_ERROR_INSTALL_REQUEST_FAILED: A install callback error.
 * @CDOS_UPDATE_ERROR_CANCEL_FAILED: A cancel callback error.
 * @CDOS_UPDATE_ERROR_SET_MODE_FAILED: A set update mode callback error.
 * 
 */

typedef enum
{
    CDOS_UPDATE_ERROR_BAD_VALUE = 0,
    CDOS_UPDATE_ERROR_INIT_FAILED,
    CDOS_UPDATE_ERROR_INIT_TIMEOUT,
    CDOS_UPDATE_ERROR_DOWNLOAD_FAILED,
    CDOS_UPDATE_ERROR_DOWNLOAD_REQUEST_FAILED,
    CDOS_UPDATE_ERROR_INSTALL_FAILED,
    CDOS_UPDATE_ERROR_INSTALL_REQUEST_FAILED,
    CDOS_UPDATE_ERROR_CANCEL_FAILED,
    CDOS_UPDATE_ERROR_SET_MODE_FAILED,
} CdosUpdateError;

/**
 * CdosUpdateMode:
 * @AUTO_UPDATE: Auto update mode.
 * @MANUAL_UPDATE: Manual update mode.
 * @AUTO_PROMPT_UPDATE: Auto prompt update mode.
 * @DEFAULT_UPDATE: System update mode.
 * 
 * The update mode of the CDOS Application.
 */
typedef enum
{
    AUTO_UPDATE,
    MANUAL_UPDATE,
    AUTO_PROMPT_UPDATE,
    DEFAULT_UPDATE,
} CdosUpdateMode;

/**
 * CdosUpdateCheckCb:
 * @version: The latest version of the application.
 * @user_data: User data.
 * @error: The error message.
 *
 * The callback of cdos_update_manager_check().
 */
typedef void (*CdosUpdateCheckCb)(const char* version, gpointer user_data, const GError* error);

/**
 * CdosUpdateProgressCb:
 * @progress: The progress value of the downloading or installing.
 * @status: The status of the downloading or installing.
 * @user_data: User data.
 * @error: The error message.
 *
 * The callback of cdos_update_manager_download() and cdos_update_manager_install().
 */
typedef void (*CdosUpdateProgressCb)(int progress, const char* status, gpointer user_data, const GError* error);

/**
 * CdosUpdateModeCb:
 * @mode: The update mode that the user set.
 * @user_data: User data.
 * @error: The error message.
 *
 * The callback of cdos_update_manager_get_mode().
 */
typedef void (*CdosUpdateModeCb)(CdosUpdateMode mode, gpointer user_data, const GError* error);

/**
 * CdosUpdateCommonResultCb:
 * @user_data: User data.
 * @error: The error message.
 *
 * The common callback of cdos_update_manager_set_mode() and cdos_update_manager_cancel().
 */
typedef void (*CdosUpdateCommonResultCb)(gpointer user_data, const GError* error);

GType cdos_update_manager_get_type (void);

/**
 * cdos_update_manager_new:
 * 
 * Create a new #CdosUpdateManager.
 * 
 * Returns: The new #CdosUpdateManager.
 */
CdosUpdateManager* cdos_update_manager_new ();

/**
 * cdos_update_manager_check:
 * @update: A #CdosUpdateManager instance.
 * @callback: The #CdosUpdateCheckCb callback function.
 * @user_data: The user data for @callback.
 * @cbdestroy: A #GDestroyNotify free function for @user_data or %NULL.
 * 
 */
void cdos_update_manager_check (CdosUpdateManager *update,
                                CdosUpdateCheckCb  callback,
                                gpointer           user_data,
                                GDestroyNotify     cbdestroy);

/**
 * cdos_update_manager_download:
 * @update: A #CdosUpdateManager instance.
 * @version: The version of package to download or %NULL. If the version is NULL, it will download the latest version.
 * @callback: The #CdosUpdateProgressCb callback function.
 * @user_data: The user data for @callback.
 * @cbdestroy: A #GDestroyNotify free function for @user_data or %NULL.
 * 
 */
void cdos_update_manager_download (CdosUpdateManager    *update,
                                   char                 *version,
                                   CdosUpdateProgressCb  callback,
                                   gpointer              user_data,
                                   GDestroyNotify        cbdestroy);

/**
 * cdos_update_manager_install:
 * @update: A #CdosUpdateManager instance.
 * @version: The version of package to install or %NULL. If the version is NULL, it will install the latest version.
 * @exit_flag: The flag of exiting the process when installing the update package. The process needs to exit: TRUE. The process does not need to exit: FALSE.
 * @callback: The #CdosUpdateProgressCb callback function.
 * @user_data: The user data for @callback.
 * @cbdestroy: A #GDestroyNotify free function for @user_data or %NULL.
 * 
 */
void cdos_update_manager_install (CdosUpdateManager      *update,
                                  char                   *version,
                                  gboolean                exit_flag,
                                  CdosUpdateProgressCb    callback,
                                  gpointer                user_data,
                                  GDestroyNotify          cbdestroy);

/**
 * cdos_update_manager_cancel:
 * @update: A #CdosUpdateManager instance.
 * @callback: The #CdosUpdateCommonResultCb callback function.
 * @user_data: The user data for @callback.
 * @cbdestroy: A #GDestroyNotify free function for @user_data or %NULL.
 * 
 */
void cdos_update_manager_cancel (CdosUpdateManager        *update,
                                 CdosUpdateCommonResultCb  callback,
                                 gpointer                  user_data,
                                 GDestroyNotify            cbdestroy);

/**
 * cdos_update_manager_set_mode:
 * @update: A #CdosUpdateManager instance.
 * @mode: The update mode.
 * @callback: The #CdosUpdateCommonResultCb callback function.
 * @user_data: The user data for @callback.
 * @cbdestroy: A #GDestroyNotify free function for @user_data or %NULL.
 * 
 */
void cdos_update_manager_set_mode (CdosUpdateManager        *update,
                                   CdosUpdateMode            mode,
                                   CdosUpdateCommonResultCb  callback,
                                   gpointer                  user_data,
                                   GDestroyNotify            cbdestroy);

/**
 * cdos_update_manager_get_mode:
 * @update: A #CdosUpdateManager instance.
 * @callback: The #CdosUpdateModeCb callback function.
 * @user_data: The user data for @callback.
 * @cbdestroy: A #GDestroyNotify free function for @user_data or %NULL.
 * 
 */
void cdos_update_manager_get_mode (CdosUpdateManager *update,
                                   CdosUpdateModeCb   callback,
                                   gpointer           user_data,
                                   GDestroyNotify     cbdestroy);

G_END_DECLS
#endif /* CDOS_UPDATE_MANAGER_H */
